@@ -122,6 +122,7 @@ class MultipleImputedKernel(MultipleImputedDataSet):
         random_state = ensure_rng(random_state)
 
         super().__init__(
+            kernel_data=data,
             initial_dataset=KernelDataSet(
                 data=data,
                 variable_schema=variable_schema,
@@ -131,7 +132,7 @@ class MultipleImputedKernel(MultipleImputedDataSet):
                 save_all_iterations=save_all_iterations,
                 save_models=save_models,
                 random_state=random_state,
-            )
+            ),
         )
 
         # Prime with the required number of kernel datasets
@@ -249,7 +250,7 @@ class MultipleImputedKernel(MultipleImputedDataSet):
         )
         if len(datasets) > 0:
             multiple_imputed_set = MultipleImputedDataSet(
-                initial_dataset=imputed_data_set
+                kernel_data=self.data, initial_dataset=imputed_data_set
             )
             while len(datasets) > 0:
                 logger.log("Dataset " + str(datasets[0]))
