@@ -14,12 +14,15 @@ setup(
     keywords=['MICE','Imputation','Missing Values','Missing','Random Forest'],
     long_description=long_description,
     long_description_content_type="text/markdown",
-    install_requires=['sklearn',
+    install_requires=['lightgbm',
                       'numpy',
                       'pandas',
                       'seaborn',
                       'matplotlib'
                       ],
+    extras_require={
+            "DMM": ["scikit-learn"],
+        },
     url="https://github.com/AnotherSamWilson/miceforest",
     packages=find_packages(exclude=["tests.*", "tests"]),
     classifiers=[
